/*
 * xMedia
 * User Interface Control
 *
 * STATUS Commands
 * 
 * 0x01 - OK -> CMD_OK
 * 	Payload data:
 * 		no data expected
 * 	Return value:
 * 		no return value	
 * 
 * 0x02 - Failure -> CMD_FAIL
 * 	Payload data:
 * 		error code
 * 	Return value:
 * 		no return value	
 * 
 * 0x03 - Busy - CMD_BUSY
 * 	Payload data:
 * 		error code
 * 	Return value:
 * 		no return value	
 * 
 * 
 * INFO Commands
 * 
 * 0x11 - Check status
 * 	Payload data:
 * 		no data expected
 * 	Return value expected:
 * 		CMD_OK 		if device is ready
 * 		CMD_BUSY 	if device is busy
 * 		timeout 	if device is busy
 * 
 * POWER MANAGEMENT Commands
 * 
 * 0x21 - Enable/Disable LCD Backlight and 5V Supply
 * 	Payload data:
 * 		0x00 disable
 * 		0x01 to 0xFF enabled
 * 		
 * 	Return value expected:
 * 		CMD_OK 		if command was executed successfull
 * 		CMD_FAIL	if command execution failed			
 * 
 * 0x22 - Set LCD Backlight current
 * 	Payload data:
 * 		0x00 	minimal current
 * 		0xFF 	maximal current
 * 		
 * 	Return value expected:
 * 		CMD_OK 		if command was executed successfull
 * 		CMD_FAIL	if command execution failed
 * 
 * 0x23 - Enable/Disable VGA Output
 * 	Payload data:
 * 		0x00 disable
 * 		0x01 to 0xFF enabled
 * 		
 * 	Return value expected:
 * 		CMD_OK 		if command was executed successfull
 * 		CMD_FAIL	if command execution failed
 * 
 * 
 * UI commands
 * 
 * 0x31 - Button OK
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x32 - Button Back
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x33 - Button GP1
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x34 - Button GP2
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x35 - Right
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * 0x36 - Left
 * 	Payload data:
 * 		no payload data	
 * 	Return value:
 * 		no return value	
 *
 * Mouse/Touchpad emulation
 *
 * 0x51 - Absolute Position
 * 	Payload data:
 * 		data[0..1]	x coordinate - unsigned integer
 *		data[2..3]	y coordinate - unsigned integer
 *		data[4]		Left button pressed
 * 		data[5]		Right button pressed
 * 	Return value:
 * 		no return value	
 *
 * 0x52 - Relative Position
 * 	Payload data:
 * 		data[0..1]	x increment - signed integer
 *		data[2..3]	y coordinate - signed integer
 *		data[4]		Left button pressed
 * 		data[5]		Right button pressed
 * 	Return value:
 * 		no return value	
 */

#include "Ui.h"
#include "CapInterface.h"
#include "../sie/Sie.h"
#include "../blib/rs232.h"
#include "../ctrl/pm.h"


unsigned char ui_sie_id = 0;
unsigned char ui_sie_bridge_id = 0;


void (*ui_pm_callback)(siePacket packet) = 0;

void uiRegisterPmPacketHandler(void (*pm_callback)(siePacket packet) )
{
	ui_pm_callback = pm_callback;
}


void uiPacketReceived(siePacket packet)
{
	if(packet.command == 0x01)
	{
		uiMasterReady = 1;
	}
	if(packet.command >= 0x20 && packet.command < 0x30)
	{
		ui_pm_callback(packet);
	}
}

void uiCapifChange(capInterfaceKey state, char value)
{
	unsigned char command = 0;

	switch(state)
	{
		case CAPIF_KEY_OK:
			command = UI_KEY_OK;
			break;
		case CAPIF_KEY_BACK:
			command = UI_KEY_BACK;
			break;
		case CAPIF_KEY_GP1:
			command = UI_KEY_GP1;
			break;
		case CAPIF_KEY_GP2:
			command = UI_KEY_GP2;
			break;
		case CAPIF_WHEEL:
			if(value == 1)
			{
				command = UI_WHEEL_FORWARD;
			}
			else
			{
				command = UI_WHEEL_BACK;
			}
			break;			
	}

	pmWakeupCall(); //New user, triggered event -> wakeup if necessary

	if(!uiSieLocked)
		sieSubmit(ui_sie_id, command, 0, 0);
}

void uiError(sieError error)
{
	
}

void uiReceiveBridge(unsigned char data)
{
	sieReceive(ui_sie_bridge_id, data);
}

void uiSubmitBridge(siePacket data)
{
	pmWakeupCall(); //New user, triggered event (from remote control) -> wakeup if necessary

	if(!uiSieLocked)
		sieSubmitPacket(ui_sie_id, data);
}

void uiReceive(unsigned char data)
{
	sieReceive(ui_sie_id, data);
}

void uiPacketSubmit(unsigned char command, unsigned char length, unsigned char *data)
{
	if(!uiSieLocked)
		sieSubmit(ui_sie_id, command, length, *data);
}

void uiSieUnlock()
{
	uiSieLocked = 0;
}

void uiInit()
{
	sieInit(); 

	capInterfaceInit(uiCapifChange);

	uiMasterReady = 0;
	uiSieLocked = 1;

	ui_sie_id 			= sieCreate(uiPacketReceived, rs232Putchar1, uiError);
	rs232Init(1, 38400, uiReceive);

	ui_sie_bridge_id	= sieCreate(uiSubmitBridge, rs232Putchar0, uiError);
	rs232Init(0, 38400, uiReceiveBridge);

}


